/*******************************************************************************
 * (c) Copyright 2016 Microsemi Corporation.  All rights reserved.
 *
 */

#include "hw_platform.h"
#include "drivers/CoreGPIO/core_gpio.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "hal/CortexM1/cortex_nvic.h"
#include "hal.h"
#include "./CMSIS/cortexm1_cfg.h"
#include "./CMSIS/system_cortexm1_cfg.h"

#define DIVISOR 256


/******************************************************************************
 * GPIO instance data. UART instance data. Base addresses
 *****************************************************************************/
gpio_instance_t g_gpio_out, g_gpio_in;
UART_instance_t g_uart0, g_uart1;



#define BAUD_VALUE                  115200

void LED_update(void);

int sema_gpio = 0;


__attribute__((__interrupt__)) void IRQ1_IRQHandler( void )
{
	sema_gpio = 1;
	GPIO_clear_all_irq_sources(&g_gpio_in, GPIO_get_irq_sources(&g_gpio_in));
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_GPIO_IN);
}
/*-------------------------------------------------------------------------*//**
 * main() function.
 */
int main()
{
	SystemCoreClockUpdate();

	UART_init(
			&g_uart0,
			UART0_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);
	UART_init(
			&g_uart1,
			UART1_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);


    /**************************************************************************
     * Initialize the CoreGPIO driver with the base address of the CoreGPIO
     * instance to use and the initial state of the outputs.
     *************************************************************************/
	/* OUTPUTS */
	GPIO_init( &g_gpio_out, COREGPIO_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	/* INPUTS */
	GPIO_init( &g_gpio_in, COREGPIO2_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	GPIO_enable_irq(&g_gpio_in, GPIO_0);
	GPIO_enable_irq(&g_gpio_in, GPIO_1);
	GPIO_enable_irq(&g_gpio_in, GPIO_2);
	GPIO_enable_irq(&g_gpio_in, GPIO_3);

	HW_set_32bit_reg(NVIC_ENABLE_REG, (NVIC_IRQ_GPIO_IN));

	UART_polled_tx_string(&g_uart0, (uint8_t *)"UART0, Hello World...\n\r");
	UART_polled_tx_string(&g_uart1, (uint8_t *)"UART1, Hello World...\n\r");

	while(1) {
		static uint32_t counter = 0;
		uint8_t key;

    	/* uart0 rx -> uart1 tx */
		if(UART_get_rx(&g_uart0, &key, 1)) {
			UART_send(&g_uart1, &key, 1);
		}
		/* uart1 rx -> uart0 tx */
		if(UART_get_rx(&g_uart1, &key, 1)) {
			UART_send(&g_uart0, &key, 1);
		}
		/* push button hit -> counter reset */
		if(sema_gpio) {
			counter = 0;
			sema_gpio = 0;
		}
		/* counter update */
		if(counter-- ==  0) {
			counter = (SystemCoreClock/DIVISOR);
			LED_update();
		}
     }
     return 0;
}

void LED_update(void)
{
	static uint8_t temp = 1;

	temp = temp^1;
	GPIO_set_output(&g_gpio_out, GPIO_0 ,temp);
	GPIO_set_output(&g_gpio_out, GPIO_1 ,!temp);
	GPIO_set_output(&g_gpio_out, GPIO_2 ,temp);
	GPIO_set_output(&g_gpio_out, GPIO_3 ,!temp);
}
